@@ -4,7 +4,7 @@ use strict;
 use warnings;
 
 use HTML::DOM::Exception qw 'INVALID_CHARACTER_ERR WRONG_DOCUMENT_ERR
-                             INUSE_ATTRIBUTE_ERR NOT_FOUND_ERR';
+                             INUSE_ATTRIBUTE_ERR NOT_FOUND_ERR SYNTAX_ERR';
 use HTML::DOM::Node 'ELEMENT_NODE';
 use Scalar::Util qw'refaddr blessed weaken';
 
@@ -17,7 +17,7 @@ require HTML::DOM::Node;
 require HTML::DOM::NodeList::Magic;
 
 our @ISA = qw'HTML::DOM::Node';
-our $VERSION = '0.031';
+our $VERSION = '0.032';
 
 
 {
@@ -348,7 +348,7 @@ sub setAttribute {
 		my $eavesdropper = &$listener_maker(
 			$self, my $name = lc $1, $_[1]
 		);
-		defined $eavesdropper and $self-> attr_event_listener(
+		defined $eavesdropper and $self-> event_handler(
 			$name, $eavesdropper
 		);
 	}
@@ -462,7 +462,7 @@ sub setAttributeNode {
 		my $eavesdropper = &$listener_maker(
 			$_[0], $name = lc $1, $_[1]->nodeValue
 		);
-		defined $eavesdropper and $_[0]-> attr_event_listener(
+		defined $eavesdropper and $_[0]-> event_handler(
 			$name, $eavesdropper
 		);
 	}
@@ -561,25 +561,68 @@ sub _attr_specified { defined shift->attr(shift) }
 
 sub click { shift->trigger_event('click') }
 
-sub innerHTML {
-	my $self = shift;
-	my $old = join '', map $_->nodeType==ELEMENT_NODE
-			? substr($_->as_HTML,0,-1)
-			: $_->data,$self->content_list
-	  if defined wantarray;
-	if(@_) {
+# used by innerHTML and insertAdjacentHTML
+sub _html_fragment_parser {
 		require HTML'DOM; # paranoia
 		(my $tb = new HTML::DOM::TreeBuilder no_magic_forms => 1)
-		  ->_set_ownerDocument($self->ownerDocument);
+		  ->_set_ownerDocument(shift->ownerDocument);
 		$tb->parse(shift);
 		$tb->eof();
 		$_->implicit(1) for $tb, $tb->content_list; # more paranoia
+		$tb;
+}
+
+sub innerHTML {
+	my $self = shift;
+	my $old = join '', map $_->nodeType==ELEMENT_NODE
+			? substr(
+			   $_->as_HTML((undef)x2,{}),0,-1
+			  )
+			: $_->data,$self->content_list
+	  if defined wantarray;
+	if(@_) {
+		my $tb = _html_fragment_parser($self,shift);
 		$self->delete_content;
 		$self->push_content($tb->guts);
 	}
 	$old;
 }
 
+{
+ my %mm # method map
+  = qw(
+   beforebegin preinsert
+   afterend    postinsert
+   afterbegin  unshift_content
+   beforeend   push_content
+  );
+
+ sub insertAdjacentHTML {
+  my $elem = shift;
+ 
+  die new HTML::DOM::Exception:: SYNTAX_ERR,
+   "$_[0]: invalid first argument to insertAdjacentHTML"
+    unless exists $mm{ my $where = lc $_[0] };
+ 
+  my $tb = _html_fragment_parser($elem,$_[1]);
+  $elem->${\$mm{$where}}(guts $tb);
+
+  ()
+ }
+ 
+ sub insertAdjacentElement {
+  my $elem = shift;
+ 
+  die new HTML::DOM::Exception:: SYNTAX_ERR,
+   "$_[0]: invalid first argument to insertAdjacentElement"
+    unless exists $mm{ my $where = lc $_[0] };
+ 
+  $elem->${\$mm{$where}}($_[1]);
+
+  ()
+ }
+}
+
 sub starttag {
 	my $self = shift;
 	my $tag = $self->SUPER::starttag(@ _);
@@ -662,21 +705,21 @@ L<HTML::DOM/CLASSES AND DOM INTERFACES>
 # ------- HTMLHtmlElement interface ---------- #
 
 package HTML::DOM::Element::HTML;
-our $VERSION = '0.031';
+our $VERSION = '0.032';
 our @ISA = 'HTML::DOM::Element';
 sub version { shift->_attr('version' => @_) }
 
 # ------- HTMLHeadElement interface ---------- #
 
 package HTML::DOM::Element::Head;
-our $VERSION = '0.031';
+our $VERSION = '0.032';
 our @ISA = 'HTML::DOM::Element';
 sub profile { shift->_attr('profile' => @_) }
 
 # ------- HTMLLinkElement interface ---------- #
 
 package HTML::DOM::Element::Link;
-our $VERSION = '0.031';
+our $VERSION = '0.032';
 our @ISA = 'HTML::DOM::Element';
 sub disabled {
 	if(@_ > 1) {
@@ -738,7 +781,7 @@ sub removeAttributeNode {
 # ------- HTMLTitleElement interface ---------- #
 
 package HTML::DOM::Element::Title;
-our $VERSION = '0.031';
+our $VERSION = '0.032';
 our @ISA = 'HTML::DOM::Element';
 # This is what I call FWP (no lexical vars):
 sub text {
@@ -753,7 +796,7 @@ sub text {
 # ------- HTMLMetaElement interface ---------- #
 
 package HTML::DOM::Element::Meta;
-our $VERSION = '0.031';
+our $VERSION = '0.032';
 our @ISA = 'HTML::DOM::Element';
 sub content   { shift->_attr('content'    => @_) }
 sub httpEquiv { shift->_attr('http-equiv' => @_) }
@@ -763,7 +806,7 @@ sub scheme    { shift->_attr('scheme'     => @_) }
 # ------- HTMLBaseElement interface ---------- #
 
 package HTML::DOM::Element::Base;
-our $VERSION = '0.031';
+our $VERSION = '0.032';
 our @ISA = 'HTML::DOM::Element';
 *href =\& HTML::DOM::Element::Link::href;
 *target =\& HTML::DOM::Element::Link::target;
@@ -771,7 +814,7 @@ our @ISA = 'HTML::DOM::Element';
 # ------- HTMLIsIndexElement interface ---------- #
 
 package HTML::DOM::Element::IsIndex;
-our $VERSION = '0.031';
+our $VERSION = '0.032';
 our @ISA = 'HTML::DOM::Element';
 sub form     { (shift->look_up(_tag => 'form'))[0] || () }
 # ~~~ Should this be the same as Select::form? I.e., should isindex ele-
@@ -781,7 +824,7 @@ sub prompt   { shift->_attr('prompt'  => @_) }
 # ------- HTMLStyleElement interface ---------- #
 
 package HTML::DOM::Element::Style;
-our $VERSION = '0.031';
+our $VERSION = '0.032';
 our @ISA = 'HTML::DOM::Element';
 *disabled = \&HTML::DOM::Element::Link::disabled;
 *media =\& HTML::DOM::Element::Link::media;
@@ -801,7 +844,7 @@ sub sheet {
 # ------- HTMLBodyElement interface ---------- #
 
 package HTML::DOM::Element::Body;
-our $VERSION = '0.031';
+our $VERSION = '0.032';
 our @ISA = 'HTML::DOM::Element';
 sub aLink      { shift->_attr( aLink      => @_) }
 sub background { shift->_attr( background => @_) }
@@ -816,6 +859,7 @@ sub attr_event_listener {
   ? $target->attr_event_listener(@_)
   : $self->SUPER::attr_event_listener(@_);
 }
+*event_handler = \&attr_event_listener;
 
 # ------- HTMLFormElement interface ---------- #
 
@@ -826,7 +870,7 @@ sub attr_event_listener {
 # ------- HTMLUListElement interface ---------- #
 
 package HTML::DOM::Element::UL;
-our $VERSION = '0.031';
+our $VERSION = '0.032';
 our @ISA = 'HTML::DOM::Element';
 sub compact { shift->_attr( compact => @_) }
 sub type { lc shift->_attr( type => @_) }
@@ -834,7 +878,7 @@ sub type { lc shift->_attr( type => @_) }
 # ------- HTMLOListElement interface ---------- #
 
 package HTML::DOM::Element::OL;
-our $VERSION = '0.031';
+our $VERSION = '0.032';
 our @ISA = 'HTML::DOM::Element';
 sub start { shift->_attr( start => @_) }
 *compact=\&HTML::DOM::Element::UL::compact;
@@ -843,28 +887,28 @@ sub start { shift->_attr( start => @_) }
 # ------- HTMLDListElement interface ---------- #
 
 package HTML::DOM::Element::DL;
-our $VERSION = '0.031';
+our $VERSION = '0.032';
 our @ISA = 'HTML::DOM::Element';
 *compact=\&HTML::DOM::Element::UL::compact;
 
 # ------- HTMLDirectoryElement interface ---------- #
 
 package HTML::DOM::Element::Dir;
-our $VERSION = '0.031';
+our $VERSION = '0.032';
 our @ISA = 'HTML::DOM::Element';
 *compact=\&HTML::DOM::Element::UL::compact;
 
 # ------- HTMLMenuElement interface ---------- #
 
 package HTML::DOM::Element::Menu;
-our $VERSION = '0.031';
+our $VERSION = '0.032';
 our @ISA = 'HTML::DOM::Element';
 *compact=\&HTML::DOM::Element::UL::compact;
 
 # ------- HTMLLIElement interface ---------- #
 
 package HTML::DOM::Element::LI;
-our $VERSION = '0.031';
+our $VERSION = '0.032';
 our @ISA = 'HTML::DOM::Element';
 *type =\& HTML::DOM::Element::Link::type;
 sub value { shift->_attr( value => @_) }
@@ -872,49 +916,49 @@ sub value { shift->_attr( value => @_) }
 # ------- HTMLDivElement interface ---------- #
 
 package HTML::DOM::Element::Div;
-our $VERSION = '0.031';
+our $VERSION = '0.032';
 our @ISA = 'HTML::DOM::Element';
 sub align { lc shift->_attr( align => @_) }
 
 # ------- HTMLParagraphElement interface ---------- #
 
 package HTML::DOM::Element::P;
-our $VERSION = '0.031';
+our $VERSION = '0.032';
 our @ISA = 'HTML::DOM::Element';
 *align =\& HTML::DOM::Element::Div::align;
 
 # ------- HTMLHeadingElement interface ---------- #
 
 package HTML::DOM::Element::Heading;
-our $VERSION = '0.031';
+our $VERSION = '0.032';
 our @ISA = 'HTML::DOM::Element';
 *align =\& HTML::DOM::Element::Div::align;
 
 # ------- HTMLQuoteElement interface ---------- #
 
 package HTML::DOM::Element::Quote;
-our $VERSION = '0.031';
+our $VERSION = '0.032';
 our @ISA = 'HTML::DOM::Element';
 sub cite { shift->_attr( cite => @_) }
 
 # ------- HTMLPreElement interface ---------- #
 
 package HTML::DOM::Element::Pre;
-our $VERSION = '0.031';
+our $VERSION = '0.032';
 our @ISA = 'HTML::DOM::Element';
 sub width { shift->_attr( width => @_) }
 
 # ------- HTMLBRElement interface ---------- #
 
 package HTML::DOM::Element::Br;
-our $VERSION = '0.031';
+our $VERSION = '0.032';
 our @ISA = 'HTML::DOM::Element';
 sub clear { lc shift->_attr( clear => @_) }
 
 # ------- HTMLBaseFontElement interface ---------- #
 
 package HTML::DOM::Element::BaseFont;
-our $VERSION = '0.031';
+our $VERSION = '0.032';
 our @ISA = 'HTML::DOM::Element';
 sub color { shift->_attr( color => @_) }
 sub face  { shift->_attr( face  => @_) }
@@ -923,7 +967,7 @@ sub size  { shift->_attr( size  => @_) }
 # ------- HTMLBaseFontElement interface ---------- #
 
 package HTML::DOM::Element::Font;
-our $VERSION = '0.031';
+our $VERSION = '0.032';
 our @ISA = 'HTML::DOM::Element';
 *color =\& HTML::DOM::Element::BaseFont::color;
 *face =\& HTML::DOM::Element::BaseFont::face;
@@ -932,7 +976,7 @@ our @ISA = 'HTML::DOM::Element';
 # ------- HTMLHRElement interface ---------- #
 
 package HTML::DOM::Element::HR;
-our $VERSION = '0.031';
+our $VERSION = '0.032';
 our @ISA = 'HTML::DOM::Element';
 *align =\& HTML::DOM::Element::Div::align;
 sub noShade  { shift->_attr( noshade  => @_) }
@@ -942,7 +986,7 @@ sub noShade  { shift->_attr( noshade  => @_) }
 # ------- HTMLModElement interface ---------- #
 
 package HTML::DOM::Element::Mod;
-our $VERSION = '0.031';
+our $VERSION = '0.032';
 our @ISA = 'HTML::DOM::Element';
 *cite =\& HTML::DOM::Element::Quote::cite;
 sub dateTime  { shift->_attr( datetime  => @_) }
@@ -950,7 +994,7 @@ sub dateTime  { shift->_attr( datetime  => @_) }
 # ------- HTMLAnchorElement interface ---------- #
 
 package HTML::DOM::Element::A;
-our $VERSION = '0.031';
+our $VERSION = '0.032';
 our @ISA = 'HTML::DOM::Element';
 sub accessKey  { shift->_attr(               accesskey  => @_) }
 *   charset    =\&HTML::DOM::Element::Link::charset           ;
@@ -1088,7 +1132,7 @@ sub search {
 # ------- HTMLImageElement interface ---------- #
 
 package HTML::DOM::Element::Img;
-our $VERSION = '0.031';
+our $VERSION = '0.032';
 our @ISA = 'HTML::DOM::Element';
 sub lowSrc  { shift->attr(               lowsrc  => @_) }
 *   name  = \&HTML::DOM::Element::Meta::name            ;
@@ -1107,7 +1151,7 @@ sub vspace   { shift->_attr(              vspace   => @_) }
 # ------- HTMLObjectElement interface ---------- #
 
 package HTML::DOM::Element::Object;
-our $VERSION = '0.031';
+our $VERSION = '0.032';
 our @ISA = 'HTML::DOM::Element';
 *form=\&HTML::DOM::Element::Select::form;
 sub code  { shift->_attr(               code  => @_) }
@@ -1132,7 +1176,7 @@ sub contentDocument{}
 # ------- HTMLParamElement interface ---------- #
 
 package HTML::DOM::Element::Param;
-our $VERSION = '0.031';
+our $VERSION = '0.032';
 our @ISA = 'HTML::DOM::Element';
 *name=\&HTML::DOM::Element::Meta::name;
 *type=\&HTML::DOM::Element::Link::type;
@@ -1142,7 +1186,7 @@ sub valueType{lc shift->_attr(valuetype=>@_)}
 # ------- HTMLAppletElement interface ---------- #
 
 package HTML::DOM::Element::Applet;
-our $VERSION = '0.031';
+our $VERSION = '0.032';
 our @ISA = 'HTML::DOM::Element';
 * align    = \ & HTML::DOM::Element::Div::align       ;
 * alt      = \ & HTML::DOM::Element::Img::alt         ;
@@ -1159,7 +1203,7 @@ sub object { shift -> _attr ( object => @_ ) }
 # ------- HTMLMapElement interface ---------- #
 
 package HTML::DOM::Element::Map;
-our $VERSION = '0.031';
+our $VERSION = '0.032';
 our @ISA = 'HTML::DOM::Element';
 sub areas { # ~~~ I need to make this cache the resulting collection obj
 	my $self = shift;
@@ -1180,7 +1224,7 @@ sub areas { # ~~~ I need to make this cache the resulting collection obj
 # ------- HTMLAreaElement interface ---------- #
 
 package HTML::DOM::Element::Area;
-our $VERSION = '0.031';
+our $VERSION = '0.032';
 our @ISA = 'HTML::DOM::Element';
 * alt       = \ & HTML::DOM::Element::Img::alt         ;
 sub coords { shift -> _attr ( coords => @_ ) }
@@ -1197,7 +1241,7 @@ sub noHref { shift -> attr ( nohref => @_ ) }
 # ------- HTMLScriptElement interface ---------- #
 
 package HTML::DOM::Element::Script;
-our $VERSION = '0.031';
+our $VERSION = '0.032';
 our @ISA = 'HTML::DOM::Element';
 * text    = \ &HTML::DOM::Element::Title::text   ;
 sub htmlFor { shift -> _attr ( for   => @_ )      }
@@ -1210,7 +1254,7 @@ sub defer   { shift -> _attr ( defer => @_ )      }
 # ------- HTMLFrameSetElement interface ---------- #
 
 package HTML::DOM::Element::FrameSet;
-our $VERSION = '0.031';
+our $VERSION = '0.032';
 our @ISA = 'HTML::DOM::Element';
 sub rows { shift -> _attr ( rows   => @_ )      }
 sub cols   { shift -> _attr ( cols => @_ )      }
@@ -1218,7 +1262,7 @@ sub cols   { shift -> _attr ( cols => @_ )      }
 # ------- HTMLFrameElement interface ---------- #
 
 package HTML::DOM::Element::Frame;
-our $VERSION = '0.031';
+our $VERSION = '0.032';
 our @ISA = 'HTML::DOM::Element';
 sub frameBorder { lc shift -> _attr ( frameBorder  => @_ )      }
 sub longDesc    { shift -> _attr ( longdesc     => @_ )      }
@@ -1238,7 +1282,7 @@ sub contentWindow {
 # ------- HTMLIFrameElement interface ---------- #
 
 package HTML::DOM::Element::IFrame;
-our $VERSION = '0.031';
+our $VERSION = '0.032';
 our @ISA = 'HTML::DOM::Element';
 *align  = \&HTML::DOM::Element::Div::align;
 *frameBorder = \&HTML::DOM::Element::Frame::frameBorder;
